@@ -39,6 +39,8 @@ import shutil as _shutil
 import traceback as _traceback
 from threading import Thread as _Thread
 import logging as _logging
+from typing import Iterable, Union
+import colemen_config as _config
 
 import ftputil as _ftputil
 from secure_delete import secure_delete as _secure_delete
@@ -64,6 +66,8 @@ import colemen_utilities.string_utils as _csu
 import colemen_utilities.dict_utils as _obj
 import colemen_utilities.file_utils as _f
 import colemen_utilities.directory_utils as _directory
+import colemen_utilities.list_utils as _arr
+import colemen_utilities.file_utils.File as _File
 
 # from colemen_utilities.string_utils.string_format import array_in_string as _csu.array_in_string
 # from colemen_utilities.string_utils.string_format import file_path as _csu.file_path
@@ -905,7 +909,7 @@ def get_drive(file_path,strip_colon=True):
         return drive_tail[0].replace(":","")
     return drive_tail[0]
 
-def get_modified_time(file_path, ftp=None):
+def get_modified_time(file_path, ftp=None,rounded:bool=True):
     '''
         get the modified from the file
 
@@ -932,11 +936,14 @@ def get_modified_time(file_path, ftp=None):
         return int(_datetime.timestamp(_datetime.utcfromtimestamp(ftp.path.getmtime(file_path))))
 
     mod_time = _os.path.getmtime(file_path)
-    mod_time = int(_datetime.fromtimestamp(mod_time).replace(tzinfo=_timezone.utc).timestamp())
-    return int(_datetime.timestamp(_datetime.fromtimestamp(mod_time)))
+    if rounded is True:
+        return int(_datetime.fromtimestamp(mod_time).replace(tzinfo=_timezone.utc).timestamp())
+    return mod_time
+    # mod_time = int(_datetime.fromtimestamp(mod_time).replace(tzinfo=_timezone.utc).timestamp())
+    # return int(_datetime.timestamp(_datetime.fromtimestamp(mod_time)))
 
 
-def get_access_time(file_path):
+def get_access_time(file_path,rounded:bool=True):
     '''
         get the access from the file
 
@@ -960,10 +967,11 @@ def get_access_time(file_path):
         `method_name`: get_modified_time
     '''
     mod_time = _os.path.getatime(file_path)
-    return int(_datetime.fromtimestamp(mod_time).replace(tzinfo=_timezone.utc).timestamp())
+    if rounded is True:
+        return int(_datetime.fromtimestamp(mod_time).replace(tzinfo=_timezone.utc).timestamp())
+    return mod_time
 
-
-def get_create_time(file_path):
+def get_create_time(file_path,rounded:bool=True):
     '''
         get the create from the file
 
@@ -987,8 +995,9 @@ def get_create_time(file_path):
         `method_name`: get_modified_time
     '''
     mod_time = _os.path.getctime(file_path)
-    return int(_datetime.fromtimestamp(mod_time).replace(tzinfo=_timezone.utc).timestamp())
-
+    if rounded is True:
+        return int(_datetime.fromtimestamp(mod_time).replace(tzinfo=_timezone.utc).timestamp())
+    return mod_time
 
 def get_ext(file_path):
     '''
@@ -1225,7 +1234,19 @@ def gen_src_path(src_base, dst_base, file_path):
     relative_path = gen_relative_path(src_base, dst_base, file_path)
     return _csu.file_path(f"{src_base}/{relative_path}")
 
-def get_files(search_path=False, **kwargs):
+def get_files(
+    search_path=False,
+    recursive:bool=True,
+    exclude:Union[str,list]=None,
+    include:Union[str,list]=None,
+    extensions:Union[str,list]=None,
+    threaded:bool=True,
+    data_include=None,
+    ftp=None,
+    include_meta_data:bool=False,
+    show_count:bool=False,
+    dicts:bool=True,
+    ):
     '''
         Get all files/data from the search_path.
 
@@ -1239,7 +1260,7 @@ def get_files(search_path=False, **kwargs):
             [`recursive`=True] {boolean}
                 If True the path is iterated recursively
 
-            [`ignore`=[]] {str|list}
+            [`exclude`=[]] {str|list}
                 A term or list or terms to ignore if the file path contains any of them.
 
             [`extensions|ext|extension`=[]] {str|list}
@@ -1248,10 +1269,10 @@ def get_files(search_path=False, **kwargs):
                 if equal to "images" it will automatically search for these extensions:
                     bmp,dds,dib,eps,gif,icns,ico,im,jpg,jpeg,jpeg 2000,msp,pcx,png,ppm,sgi,spider,tga,tiff,webp,xbm
 
-            [`show count`=False] {bool}
+            [`show_count`=False] {bool}
                 If True, the index count is printed to the terminal during indexing.
 
-            [`dicts`=True] {bool}
+            [`get_files`=True] {bool}
                 if True, this will return a list of dictionaries, otherwise it will return a list of File objects.
 
             [`threaded`=True] {bool}
@@ -1299,17 +1320,17 @@ def get_files(search_path=False, **kwargs):
     if isinstance(search_path, list) is False:
         search_path = [search_path]
 
-    threaded = _obj.get_kwarg(['threaded', 'thread'], True, bool, **kwargs)
-    recursive = _obj.get_kwarg(['recursive', 'recurse'], True, bool, **kwargs)
-
-    data_include = _obj.get_kwarg(['data include'], [], (list, str), **kwargs)
-    if isinstance(data_include, (str)):
-        data_include = [data_include]
+    # threaded = _obj.get_kwarg(['threaded', 'thread'], True, bool, **kwargs)
+    # recursive = _obj.get_kwarg(['recursive', 'recurse'], True, bool, **kwargs)
+    data_include = _arr.force_list(data_include,allow_nulls=False)
+    # data_include = _obj.get_kwarg(['data include'], [], (list, str), **kwargs)
+    # if isinstance(data_include, (str)):
+        # data_include = [data_include]
 
     if len(data_include) == 0:
         data_include = _GET_DATA_INCLUDE_DEFAULT_VALUE
 
-    include_meta_data = _obj.get_kwarg(['image meta data', 'meta data','include meta data'], False, bool, **kwargs)
+    # include_meta_data = _obj.get_kwarg(['image meta data', 'meta data','include meta data'], False, bool, **kwargs)
     if include_meta_data is True:
         if "extension" not in data_include:
             data_include.append("extension")
@@ -1317,24 +1338,28 @@ def get_files(search_path=False, **kwargs):
 
 
     # ignore_array = _obj.get_kwarg(['ignore', 'ignore_array', 'exclude'], [], (str, list), **kwargs)
-    exclude = _obj.get_kwarg(['exclude', 'ignore', 'ignore array'], [], (list, str), **kwargs)
+    exclude = _arr.force_list(exclude,allow_nulls=False)
+    # exclude = _obj.get_kwarg(['exclude', 'ignore', 'ignore array'], [], (list, str), **kwargs)
     if isinstance(exclude, (str)):
         exclude = [exclude]
 
-    include = _obj.get_kwarg(['include'], [], (list, str), **kwargs)
+    include = _arr.force_list(include,allow_nulls=False)
+    # include = _obj.get_kwarg(['include'], [], (list, str), **kwargs)
     if isinstance(include, (str)):
         include = [include]
 
-    show_index_count = _obj.get_kwarg(['show_index_count', 'show count'], False, bool, **kwargs)
+    show_index_count = show_count
+    # show_index_count = _obj.get_kwarg(['show_index_count', 'show count'], False, bool, **kwargs)
 
     # print(f"file.get_files.kwarg['extension']: ",_obj.get_kwarg(['extensions', 'ext', 'extension'], [], (str, list), **kwargs))
-    extension_array = _obj.get_kwarg(['extensions', 'ext', 'extension'], [], (str, list), **kwargs)
+    extension_array = _arr.force_list(extensions,allow_nulls=False)
+    # extension_array = _obj.get_kwarg(['extensions', 'ext', 'extension'], [], (str, list), **kwargs)
     # print(f"extension_array - RAW: {extension_array}")
     extension_array = _gen_extension_array(extension_array)
     # print(f"extension_array: {extension_array}")
     # print(f"file.get_files: {search_path}")
 
-    ftp = _obj.get_kwarg(["ftp"], None, None, **kwargs)
+    # ftp = _obj.get_kwarg(["ftp"], None, None, **kwargs)
     if ftp is not None:
         return get_files_ftp(search_path, **kwargs)
 
@@ -1349,7 +1374,10 @@ def get_files(search_path=False, **kwargs):
                 extensions=extension_array,
                 include_meta_data=include_meta_data,
                 show_index_count=show_index_count)
-            return gft.master()
+            file_array = gft.master()
+            if dicts is False:
+                file_array = [_File.File(x['file_path'],x) for x in file_array]
+            return file_array
     # print(_json.dumps(extension_array, indent=4))
     for path in search_path:
         path = _csu.file_path(path)
@@ -1397,10 +1425,214 @@ def get_files(search_path=False, **kwargs):
         # file_array = path_files + file_array
     if include_meta_data is True:
         file_array = _add_jpg_meta_data(file_array)
+
     if show_index_count is True:
         print(f"Total Files Indexed: {len(file_array)}                                             ")
+
+    if dicts is False:
+        file_array = [_File.File(x['file_path'],x) for x in file_array]
+
     return file_array
 
+
+def get_files_obj(
+    search_path=False,
+    recursive:bool=True,
+    exclude:Union[str,list]=None,
+    include:Union[str,list]=None,
+    extensions:Union[str,list]=None,
+    threaded:bool=True,
+    data_include=None,
+    ftp=None,
+    include_meta_data:bool=False,
+    show_count:bool=False,
+    dicts:bool=True,
+    )->Iterable[_config._file_type]:
+
+    '''
+        Get all files/data from the search_path.
+
+        ----------
+
+        Keyword Arguments
+        -----------------
+            [`search_path`=cwd] {str|list}
+                The search path or list of paths to iterate.
+
+            [`recursive`=True] {boolean}
+                If True the path is iterated recursively
+
+            [`exclude`=[]] {str|list}
+                A term or list or terms to ignore if the file path contains any of them.
+
+            [`extensions|ext|extension`=[]] {str|list}
+                An extension or list of extensions that the file must have.\n
+                Can have leading periods or not.\n
+                if equal to "images" it will automatically search for these extensions:
+                    bmp,dds,dib,eps,gif,icns,ico,im,jpg,jpeg,jpeg 2000,msp,pcx,png,ppm,sgi,spider,tga,tiff,webp,xbm
+
+            [`show_count`=False] {bool}
+                If True, the index count is printed to the terminal during indexing.
+
+            [`get_files`=True] {bool}
+                if True, this will return a list of dictionaries, otherwise it will return a list of File objects.
+
+            [`threaded`=True] {bool}
+                if True, the process is multi-threaded,
+                this makes indexing much faster,
+                but it can easily overwhelm a cpu depending upon the drive.
+
+            `data_include` {str|list}
+                The data to get for each file, the shorter this list the faster it will complete.
+                By default it will get this:
+                    ['file_name', 'extension', 'name_no_ext', 'dir_path', 'access_time',
+                    'modified_time', 'created_time', 'size']
+
+                so the list you provide will limit the amount of
+                reading/formatting needed to gather data.
+
+                Example:
+                    ['modified_time','size']  will take ~0.000129939 seconds per file on an SSD
+
+                    ['file_name', 'extension', 'name_no_ext', 'dir_path', 'access_time',
+                    'modified_time', 'created_time', 'size']
+
+                    will take ~0.000173600 seconds per file on an SSD,
+                    it's not much but little things matter.. that's what she said.
+
+            [`ftp`=None] {obj}
+                A reference to the ftputil object.
+
+            [`include_meta_data`=False] {bool}
+                If True any images that are found will have their meta_data added to the file object.
+
+                Bare in mind that not all images support keywords etc. Currently, only jpg and tiff allow it.
+
+
+                This will slow things down a bit.
+
+        return
+        ----------
+        `return` {list}
+            A list of dictionaries containing all matching files.
+    '''
+    file_array = []
+    if search_path is False:
+        search_path = _obj.get_kwarg(['search path', 'search'], _os.getcwd(), (str, list), **kwargs)
+    if isinstance(search_path, list) is False:
+        search_path = [search_path]
+
+    # threaded = _obj.get_kwarg(['threaded', 'thread'], True, bool, **kwargs)
+    # recursive = _obj.get_kwarg(['recursive', 'recurse'], True, bool, **kwargs)
+    data_include = _arr.force_list(data_include,allow_nulls=False)
+    # data_include = _obj.get_kwarg(['data include'], [], (list, str), **kwargs)
+    # if isinstance(data_include, (str)):
+        # data_include = [data_include]
+
+    if len(data_include) == 0:
+        data_include = _GET_DATA_INCLUDE_DEFAULT_VALUE
+
+    # include_meta_data = _obj.get_kwarg(['image meta data', 'meta data','include meta data'], False, bool, **kwargs)
+    if include_meta_data is True:
+        if "extension" not in data_include:
+            data_include.append("extension")
+
+
+
+    # ignore_array = _obj.get_kwarg(['ignore', 'ignore_array', 'exclude'], [], (str, list), **kwargs)
+    exclude = _arr.force_list(exclude,allow_nulls=False)
+    # exclude = _obj.get_kwarg(['exclude', 'ignore', 'ignore array'], [], (list, str), **kwargs)
+    if isinstance(exclude, (str)):
+        exclude = [exclude]
+
+    include = _arr.force_list(include,allow_nulls=False)
+    # include = _obj.get_kwarg(['include'], [], (list, str), **kwargs)
+    if isinstance(include, (str)):
+        include = [include]
+
+    show_index_count = show_count
+    # show_index_count = _obj.get_kwarg(['show_index_count', 'show count'], False, bool, **kwargs)
+
+    # print(f"file.get_files.kwarg['extension']: ",_obj.get_kwarg(['extensions', 'ext', 'extension'], [], (str, list), **kwargs))
+    extension_array = _arr.force_list(extensions,allow_nulls=False)
+    # extension_array = _obj.get_kwarg(['extensions', 'ext', 'extension'], [], (str, list), **kwargs)
+    # print(f"extension_array - RAW: {extension_array}")
+    extension_array = _gen_extension_array(extension_array)
+    # print(f"extension_array: {extension_array}")
+    # print(f"file.get_files: {search_path}")
+
+    # ftp = _obj.get_kwarg(["ftp"], None, None, **kwargs)
+    if ftp is not None:
+        return get_files_ftp(search_path, **kwargs)
+
+    if threaded is True:
+        # print(f"file.get_files - Using threads for indexing.")
+        for path in search_path:
+            gft = GetFilesThreaded(path,
+                recursive=recursive,
+                data_include=data_include,
+                exclude=exclude,
+                include=include,
+                extensions=extension_array,
+                include_meta_data=include_meta_data,
+                show_index_count=show_index_count)
+            file_array = gft.master()
+            return [_File.File(x['file_path'],x) for x in file_array]
+    # print(_json.dumps(extension_array, indent=4))
+    for path in search_path:
+        path = _csu.file_path(path)
+        # pylint: disable=unused-variable
+        for root, folders, files in _os.walk(path):
+            for file in files:
+                f = _File.File(_os.path.join(root, file))
+                # file_data = get_data(_os.path.join(root, file), include=data_include)
+            # if file_data is not None:
+                # ignore = False
+                # print(f"file_data['extension']: {file_data['extension']}")
+                if len(exclude) > 0:
+                    # print(f"filtering excludes")
+                    if _csu.array_in_string(exclude, f.file_path) is True:
+                        continue
+
+                if len(include) > 0:
+                    # print(f"filtering includes")
+                    if _csu.array_in_string(include, f.file_path) is False:
+                        continue
+
+                if len(extension_array) > 0:
+                    # print(f"filtering extension_array")
+                    if f.extension not in extension_array:
+                        continue
+
+                # if len(ignore_array) > 0:
+                #     for ignore_string in ignore_array:
+                #         if ignore_string in file_data['file_path']:
+                #             ignore = True
+
+                # if ignore is False:
+                    # fd['file_hash'] = generateFileHash(fd['file_path'])
+                # print(f"file found.")
+                file_array.append(f)
+                if show_index_count is True:
+                    print(f"files indexed: {len(file_array)}                                             ",end="\r",flush=True)
+                # else:
+                    # print(f"file_data is none")
+            if recursive is False:
+                # print(f"breaking")
+                break
+
+        # path_files = index_files(path, extension_array, ignore_array, recursive)
+        # file_array = path_files + file_array
+    # if include_meta_data is True:
+    #     file_array = _add_jpg_meta_data(file_array)
+
+    if show_index_count is True:
+        print(f"Total Files Indexed: {len(file_array)}                                             ")
+
+
+    return file_array
+
+
 def _add_jpg_meta_data(file_array):
     '''
         Gathers meta data for jpg files.
@@ -1876,6 +2108,13 @@ class GetFilesThreaded:
         #     # print(f"Total files indexed: {len(file_array)}")
 
 
+
+def get_file(file_path:str)->_config._file_type:
+    # if exists(file_path) is False:
+        # return False
+    return _File.File(file_path)
+
+
 # file = r"C:\Users\Colemen\Desktop\DAZDOW~1\poses\STANDI~1\IM0008~1\Content\People\GENESI~2\Poses\AEONSO~1\STANDI~1\LIMBSL~1\SC-WE'~2.DUF"
 # file = r"C:\\Users\\Colemen\\Desktop\\DAZ DOWNLOADS\\poses\\Standing Conversation Poses for Genesis 8\\IM00083571-01_StandingConversationPosesforGenesis8\\Content\\People\\Genesis 8 Male\\Poses\\Aeon Soul\\Standing Conversation\\Limbs Legs\\SC-We're all in the same boat Legs-M Genesis 8 Male.duf"
 # clean = clean_path(file)
