@@ -34,7 +34,7 @@ import colemen_utilities.string_utils as _csu
 import colemen_utilities.file_utils as _f
 import colemen_utilities.sql_utils as _sql
 import colemen_utilities.console_utils as _console
-import colemen_utilities.sql_utils.TableStatement as _table_statement
+# import colemen_utilities.sql_utils.TableStatement as _table_statement
 # import colemen_config as _config
 _log = _console.log
 
@@ -1304,53 +1304,53 @@ def format_sql_string(sql):
 
 
 
-def parse_new(sql):
-    # print("        sql_parse.parse")
-    if _f.exists(sql):
-        # print(f"    Reading SQL file: {sql}")
-        sql = _f.readr(sql)
+# def parse_new(sql):
+#     # print("        sql_parse.parse")
+#     if _f.exists(sql):
+#         # print(f"    Reading SQL file: {sql}")
+#         sql = _f.readr(sql)
 
-    if len(sql)== 0:
-        return None
+#     if len(sql)== 0:
+#         return None
 
-    sql = _csu.strip_empty_lines(sql)
-    raw_statements = get_statements(sql)
-    # print(f"raw_statements: {len(raw_statements)}")
-    data = {
-        "schemas":[],
-        "tables":[],
-        "statements":[],
-    }
+#     sql = _csu.strip_empty_lines(sql)
+#     raw_statements = get_statements(sql)
+#     # print(f"raw_statements: {len(raw_statements)}")
+#     data = {
+#         "schemas":[],
+#         "tables":[],
+#         "statements":[],
+#     }
 
-    for s in raw_statements:
-        s = s.value
-        # print(f"s: {s}")
-        state = {
-            "raw":s,
-            "purpose":determine_statement_purpose(s),
-            "data":None,
-        }
+#     for s in raw_statements:
+#         s = s.value
+#         # print(f"s: {s}")
+#         state = {
+#             "raw":s,
+#             "purpose":determine_statement_purpose(s),
+#             "data":None,
+#         }
 
-        if state['purpose'] is None:
-            _log(f"    Failed to determine purpose of statement:\n {s}","error")
-            continue
+#         if state['purpose'] is None:
+#             _log(f"    Failed to determine purpose of statement:\n {s}","error")
+#             continue
 
-        if state['purpose'] == "CREATE TABLE":
-            # _log(f"    Create Table Statement found statement:\n {s}","success")
-            table = _table_statement.TableStatement(state['raw'])
-            table.parse()
-            state['data'] = table.summary
-            # state['data'] = parse_table_statement(state['raw'])
-            _lu.append(data['tables'],state['data'])
-            continue
+#         if state['purpose'] == "CREATE TABLE":
+#             # _log(f"    Create Table Statement found statement:\n {s}","success")
+#             table = _table_statement.TableStatement(state['raw'])
+#             table.parse()
+#             state['data'] = table.summary
+#             # state['data'] = parse_table_statement(state['raw'])
+#             _lu.append(data['tables'],state['data'])
+#             continue
 
-        if state['purpose'] == "CREATE SCHEMA":
-            state['data'] = parse_schema_statement(state['raw'])
-            _lu.append(data['schemas'],state['data'])
-            continue
+#         if state['purpose'] == "CREATE SCHEMA":
+#             state['data'] = parse_schema_statement(state['raw'])
+#             _lu.append(data['schemas'],state['data'])
+#             continue
 
-        data['statements'].append(state)
+#         data['statements'].append(state)
 
-    data = _parse_data_comments(data)
-    return data
+#     data = _parse_data_comments(data)
+#     return data
 
